/*
 * Created on 29-03-2003
 *
 */
package net.sf.hibernate.console;

import java.awt.Component;
import java.awt.Container;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.WeakHashMap;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.basic.BasicSplitPaneDivider;
import javax.swing.plaf.basic.BasicSplitPaneUI;
import javax.swing.tree.DefaultTreeModel;

import net.sf.hibernate.HibernateException;
import net.sf.hibernate.Session;
import net.sf.hibernate.SessionFactory;
import net.sf.hibernate.cfg.Configuration;
import net.sf.hibernate.console.ConfigurationUI.ConsoleConfiguration;
import net.sf.hibernate.console.docking.ElegantDockingPort;
import net.sf.hibernate.console.docking.ElegantPanel;
import net.sf.hibernate.console.node.BaseNode;
import net.sf.hibernate.console.node.NodeFactory;
import net.sf.hibernate.console.views.GraphView;
import net.sf.hibernate.console.views.ObjectGraphView;
import net.sf.hibernate.console.views.ObjectPropertySheetView;
import net.sf.hibernate.console.views.QueryCommanderView;
import net.sf.hibernate.console.views.QueryPageView;
import net.sf.hibernate.console.views.SessionFactoryView;
import net.sf.hibernate.console.views.SessionView;
import net.sf.hibernate.console.views.StaticModelView;
import net.sf.hibernate.console.views.View;
import net.sf.hibernate.tool.hbm2ddl.SchemaExport;
import net.sf.hibernate.util.ReflectHelper;

import org.pf.joi.Inspector;

import com.jgoodies.clearlook.ClearLookManager;
import com.jgoodies.clearlook.ClearLookMode;
import com.jgoodies.plaf.plastic.PlasticXPLookAndFeel;

/**
 * Main class for Hibernate Console. Either use directly by runing main() or do a Start.startWith(configuration);
 * 
 * @author max
 *
 */
public class Start implements QueryExecutor, SessionController {

	JSplitPane horizontalSplit;
	JSplitPane vertSplitLeft;
	JSplitPane vertSplitRight;
	
	Map fakeDrivers = new HashMap();
	Map views = new HashMap();
	int execcount;
	List sessionListeners = new ArrayList();
	List factoryListeners = new ArrayList();
	
	final static String dockingSettingsFileName = "hibernateconsole-docking.xml";
	private JMenu namedQueries;
	private ErrorListModel errorList;
	private JFrame mainWindow;
	Configuration configuration;
	SessionFactory sf;

	private Action executeAction;
    
	
	private URLClassLoader configurationClassLoader;
	private Map previousLoaders = new WeakHashMap();
	private QueryCommanderView queryCommander;
	    
    Start() {
		
	}

	void reconfig(ConsoleConfiguration config) {
		File configfile = config.getXmlconfig();
		Properties props = config.getProperties();
		List mappings = config.getMappingFiles();
		List paths = config.getCustomClasspath();
		File hbm = null;
		try {
			
			
			Iterator maps = paths.iterator();
			
			List urls = new ArrayList();
			
			while (maps.hasNext()) {
				File dir = (File) maps.next();
				urls.add(dir.toURL());							
			}

			if(urls.size()>0) {
                configurationClassLoader = new URLClassLoader((URL[]) urls.toArray(new URL[0]), Thread.currentThread().getContextClassLoader());
                installLoader();			
			}
			
			configuration = new Configuration();
        
            
			configuration = configuration.setProperties(props);

			if (configfile != null) {
				configuration = configuration.configure(configfile);
			}

			Iterator hbms = mappings.iterator();
		
			while (hbms.hasNext()) {
				hbm = (File) hbms.next();
				configuration = configuration.addFile(hbm.toString());
			}

			//	convince DriverManager that you can use our specified driver!
			String driverClassName = configuration.getProperty("hibernate.connection.driver_class");
			if(driverClassName!=null) {
				try {
					Class driverClass = ReflectHelper.classForName(driverClassName);
					if(!fakeDrivers.containsKey(driverClassName)) { // To avoid "double registration"
						FakeDelegatingDriver fakeDelegatingDriver = new FakeDelegatingDriver((Driver) driverClass.newInstance());
						DriverManager.registerDriver(fakeDelegatingDriver);
						fakeDrivers.put(driverClassName,fakeDelegatingDriver);
					}
					
				} catch (ClassNotFoundException e1) {
					reportException(e1);					
				} catch (InstantiationException e) {
					reportException(e);
				} catch (IllegalAccessException e) {
					reportException(e);
				} catch (SQLException e) {
					reportException(e);	
				}
			}
			
			initialize();
		} catch (HibernateException e) {
			reportException(e);  //TODO: report which hbm file is doing the error.
		} catch (MalformedURLException e) {
			reportException(e);			
		} finally {
			uninstallLoader();
		}
	}

	

	/**
	 * @param thread
	 * @param prevCl
	 */
	private void uninstallLoader() {
		ClassLoader cl = (ClassLoader) previousLoaders.get(Thread.currentThread());
		if(cl!=null) {
			Thread.currentThread().setContextClassLoader(cl);
		}
	}

	private void initialize() throws HibernateException {
		try {
			installLoader();
			if (configuration != null) {
				sf = configuration.buildSessionFactory();
			
				//registerMBean();				
				//JGraphUtilities.applyLayout(graph, new AnnealingLayoutAlgorithm());
				buildMenuOfAvailableQueries();
				fireFactoryCreated(configuration, sf);
		}
		} finally {
			uninstallLoader();
		}
	}

	private void fireSessionCreated(Session session) {
		Iterator i = sessionListeners.iterator();
		while (i.hasNext()) {
			SessionView view = (SessionView) i.next();
			view.sessionOpened(session);
		}			
	}
	private void fireQueryPageCreated(QueryPage qp) {
		Iterator i = sessionListeners.iterator();
		while (i.hasNext()) {
			SessionView view = (SessionView) i.next();
			view.queryPageCreated(qp);
		}		
	}
	protected void fireObjectUpdated(Session session, Object o) {
		Iterator i = sessionListeners.iterator();
		while (i.hasNext()) {
			SessionView view = (SessionView) i.next();
			view.objectUpdated(session, o);
		}		
	}

	private void fireFactoryCreated(Configuration cfg, SessionFactory mySf) {
		Iterator i = factoryListeners.iterator();
		while (i.hasNext()) {
			SessionFactoryView view = (SessionFactoryView) i.next();
			view.factoryCreated(cfg, sf);
		}
	}
	
	private void fireFactoryUpdated(SessionFactory sf) {
		Iterator i = factoryListeners.iterator();
		while (i.hasNext()) {
			SessionFactoryView view = (SessionFactoryView) i.next();
			view.factoryUpdated(sf);
		}
	}

	/**
	 * 
	 */
	private void installLoader() {
		if(configurationClassLoader!=null) {
			previousLoaders.put(Thread.currentThread(), Thread.currentThread().getContextClassLoader());
			Thread.currentThread().setContextClassLoader(configurationClassLoader);
		}		
	}

			
		public static void startWith(Configuration c, boolean lookGood) throws HibernateException {
			if (lookGood) setupSwing();
			Start h8ide = new Start();
			h8ide.setConfiguration(c);
			h8ide.start();
		}
	
	public static void startWith(Configuration c) throws HibernateException {
		startWith(c, false);
	}

	public static void main(String[] args) throws HibernateException {
		startWith(null,true);
	}
	
	private static void setupSwing() {
		try {
			if (System.getProperty("sun.awt.exception.handler")==null) {
				System.setProperty("sun.awt.exception.handler", AWTExceptionHandler.class.getName());			
			} else {
				System.err.println("could not install AWT handler ;(");
			}
			ClearLookManager.setMode(ClearLookMode.ON);
			UIManager.setLookAndFeel(new PlasticXPLookAndFeel());
			//UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			
		} catch (Exception e) { // noop 
			
		}
	}

	/**
	 * @param cfg
	 */
	private void setConfiguration(Configuration cfg) {
		configuration = cfg;

	}

	private void start() throws HibernateException {
	
		mainWindow = setupUI();
		
		mainWindow.show();
		
		horizontalSplit.setDividerLocation(0.3d);
		vertSplitLeft.setDividerLocation(0.75d);
		vertSplitRight.setDividerLocation(0.75d);				
	}

	private JFrame setupUI() throws HibernateException {
		ElegantDockingPort bottomRight;
		ElegantDockingPort topRight;
		ElegantDockingPort bottomLeft;
		ElegantDockingPort topLeft;
		
		final JFrame main = new JFrame("Hibernate Console 2.1");
		main.setIconImage(Toolkit.getDefaultToolkit().getImage(Start.class.getResource("images/hibernate_icon.jpg")));			

		createActions();
		buildViews();
		
		horizontalSplit = createSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		vertSplitLeft = createSplitPane(JSplitPane.VERTICAL_SPLIT);
		vertSplitRight = createSplitPane(JSplitPane.VERTICAL_SPLIT);
		
		
		horizontalSplit.setLeftComponent(vertSplitLeft);
		horizontalSplit.setRightComponent(vertSplitRight);
		topLeft = new ElegantDockingPort("top.left");
		bottomLeft = new ElegantDockingPort("bottom.left");
		topRight = new ElegantDockingPort("top.right");
		bottomRight = new ElegantDockingPort("bottom.right");
		
		topLeft.add((ElegantPanel) views.get("static.graph"));
		topLeft.add((ElegantPanel) views.get("static.tree"));		
		
		bottomLeft.add((ElegantPanel) views.get("messages"));
		bottomLeft.add((ElegantPanel) views.get("dynamic.graph"));
		bottomLeft.add((ElegantPanel) views.get("properties"));
		
		
		topRight.add((ElegantPanel) views.get("query"));
				
		bottomRight.add((ElegantPanel) views.get("result"));
		
		
		vertSplitLeft.setLeftComponent(topLeft);
		vertSplitLeft.setRightComponent(bottomLeft);
		vertSplitRight.setLeftComponent(topRight);
		vertSplitRight.setRightComponent(bottomRight);
				
		horizontalSplit.setLeftComponent(vertSplitLeft);
		horizontalSplit.setRightComponent(vertSplitRight);		
		
		main.setContentPane(horizontalSplit);
				

		JMenuBar bar = buildMenuBar();

		main.setJMenuBar(bar);
		main.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {				
				super.windowClosing(e);
				/*DockingConfiguration dockCfg = ConfigurationManager.createDockingConfiguration();
				ConfigurationManager.store(dockCfg, new File(dockingSettingsFileName));*/
			}
		});
		main.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		main.setSize(800, 600);
		//main.setExtendedState(Frame.MAXIMIZED_BOTH);
		
		
		/*File dockingSettingsFile = new File(dockingSettingsFileName);
		if(dockingSettingsFile.exists() && dockingSettingsFile.canRead() && dockingSettingsFile.isFile()) {			
			DockingConfiguration config = ConfigurationManager.load(dockingSettingsFile);
			ConfigurationManager.applyConfiguration(config);
		}
		*/
		initialize();				
	
		return main;
	}


    
	private void buildViews() {
		views.put("static.graph", new ElegantPanel("static.graph","Static graph",createStrippedScrollPane(buildConfigGraph().getContainer())));
		views.put("static.tree",new ElegantPanel("static.tree","Static tree",createStrippedScrollPane(buildConfigTree().getContainer())));
		views.put("messages", new ElegantPanel("messages","Messages", buildMessagePanel()));
		views.put("dynamic.graph",new ElegantPanel("dynamic.graph","Object graph", createStrippedScrollPane(buildObjectGraph().getContainer())));
		views.put("properties", new ElegantPanel("properties","Properties", buildPropertySheet().getContainer()));
		views.put("query", new ElegantPanel("query","Query", createStrippedScrollPane(buildHQLCommander().getContainer())));


		views.put("result", new ElegantPanel("result","Result", createStrippedScrollPane(buildQueryPageViewer().getContainer())));
	}

	private JMenuBar buildMenuBar() {
		JMenuBar bar = new JMenuBar();

		JMenu menu = new JMenu("Configuration");
		menu.add(new ConfigurationAction());
		menu.add(new RunSchemaExport());
		menu.add(new AbstractAction("Reinitialize") {
			public void actionPerformed(ActionEvent e) {
				
				try {
					initialize();
				} catch (HibernateException e1) {
					reportException(e1);
				}
			}
		});
		final JCheckBoxMenuItem item = new JCheckBoxMenuItem();
		item.setAction(new AbstractAction("Show full class names") {
			public void actionPerformed(ActionEvent e) {
				
				MappingsTreeCellRenderer.setUseFullyQualifiedNames(item.isSelected());
			}
		});
		menu.add(item);
		
		
		bar.add(menu);
		
		menu = new JMenu("Query");

		menu.add(executeAction);
		menu.add(buildMenuOfAvailableQueries());
		
		bar.add(menu);
		return bar;
	}

	private void createActions() {
		executeAction = new ExecuteQuery();        			
	}

	/**
	 * @return
	 */
	private JMenuItem buildMenuOfAvailableQueries() {
		if (namedQueries == null) {
			namedQueries = new JMenu("Named queries");
		}
		namedQueries.removeAll();
		Map map = null;
		if (configuration == null) {
			JMenuItem empty = new JMenuItem("<<No Configuration>>");
			empty.setEnabled(false);
			namedQueries.add(empty);
		} else {
			map = configuration.getNamedQueries();
			Iterator keys = map.keySet().iterator();
			while (keys.hasNext()) {
				String key = (String) keys.next();
				String queryString = (String) map.get(key);
				namedQueries.add(new NamedQueryAction(key, queryString));
			}
		}

		if (map != null && map.size() == 0) {
			namedQueries.add(new JMenuItem("<<No named Queries>>"));
		}

		return namedQueries;
	}

	class NamedQueryAction extends AbstractAction {

		String query;
		NamedQueryAction(String name, String query) {
			super(name);
			this.query = query.trim();
		}

		public void actionPerformed(ActionEvent e) {
			executeQuery(query);
		}
	}

	/**
	 * @return
	 */
	private View buildConfigGraph() {
		SessionFactoryView v = new GraphView();
		addListener(v);
		return v;
	}

	
	private void addListener(SessionFactoryView v) {
		factoryListeners.add(v);		
	}

	/**
	 * @return
	 */
	private SessionFactoryView buildConfigTree() {
		SessionFactoryView v = new StaticModelView(this);
		addListener(v);
		return v;
	}

	/**
	 * @param node
	 */
	public void executeQuery(String hql) {
		queryCommander.setQueryText(hql);
	}

	/**
	 * @return
	 */
	private Container buildMessagePanel() {
		JList list = new JList();
		errorList = new ErrorListModel();
		list.setModel(errorList);
		list.setCellRenderer(new ErrorCellRenderer());		
		return createStrippedScrollPane(list);
	}

	/**
	 * @return
	 */
	private Container buildConfigPanel() {

		JPanel p = new ConfigurationUI(new ConfigurationUI.ConfigurationUIListener() {
			public void configurationChanged(ConsoleConfiguration config) {
				reconfig(config);
			}

		});

		return p;

	}

	/**
	 * @return
	 */
	private SessionView buildHQLCommander() {
		queryCommander = new QueryCommanderView(this);
		addSessionListener(queryCommander);
		return queryCommander;
	}

	/**
	 * @param queryPages
	 * @return
	 */
	private SessionView buildQueryPageViewer() {
		SessionView v = new QueryPageView(this,errorList,this);
		addSessionListener(v);
		return v;
	}

	Map formParts = new HashMap();
			
	private void reportException(Throwable ef) {
		ef.printStackTrace();
		errorList.addError(ef);
	}

	private void reportInfo(String string) {
		errorList.addInfo(string);
	}

	public class ConfigurationAction extends AbstractAction {
		
		public ConfigurationAction() {
			super("Configure...");
		}
		
		public void actionPerformed(ActionEvent e) {
			JDialog d = new JDialog(mainWindow);
			
			d.getContentPane().add(buildConfigPanel());
			d.pack();
			
			d.show();

		}
	}
	public class RunSchemaExport extends AbstractAction {
		RunSchemaExport() {
			super("Run schema export");
		}

		public void actionPerformed(ActionEvent e) {
			try {
				installLoader();
				if(configuration!=null) {
					int result = JOptionPane.showConfirmDialog(mainWindow, "Are you sure you want to (re)create the database schema ? All data can get lost!", "Create database schema", JOptionPane.YES_NO_OPTION);
					if (result == JOptionPane.YES_OPTION) {
						new SchemaExport(configuration).create(false, true);
					}
				}
			} catch (HibernateException ef) {

				reportException(ef);
			} finally {
				uninstallLoader();
			}

		}

	}

	/**
	     * @author max
	     *
	     */
	public class ExecuteQuery extends AbstractAction {
		int execcount = 0;
		ExecuteQuery() {
			super("Execute query");
			//initialize the action fields with what you want
			putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, InputEvent.CTRL_DOWN_MASK));
			putValue(ACTION_COMMAND_KEY, "execQuery");
			putValue(LONG_DESCRIPTION, "Execute the HQL Query");
			putValue(MNEMONIC_KEY, new Integer('q'));
			putValue(SHORT_DESCRIPTION, "Execute Query");
			//putValue(SMALL_ICON, icon);
		}

		public void actionPerformed(ActionEvent e) {

			executeHQLQuery(queryCommander.getQueryText());
		}		
	}

    public class ExecuteJava extends AbstractAction {
        int execcount = 0;
        ExecuteJava() {
            super("Execute Java");
            //initialize the action fields with what you want
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, InputEvent.SHIFT_DOWN_MASK));
            putValue(ACTION_COMMAND_KEY, "execJava");
            putValue(LONG_DESCRIPTION, "Execute Java code and show the result");
            putValue(MNEMONIC_KEY, new Integer('j'));
            putValue(SHORT_DESCRIPTION, "Execute Java");
            //putValue(SMALL_ICON, icon);
        }

        public void actionPerformed(ActionEvent e) {
        	executeJavaQuery(queryCommander.getQueryText());        
        }
    }

    /**
     * Creates and answers a <code>JScrollpane</code> that has no border.
     */
    public static JScrollPane createStrippedScrollPane(Component c) {
    	JScrollPane scrollPane = new JScrollPane(c);
    	scrollPane.setBorder(null);
    	return scrollPane;
    }
    
    private static JSplitPane createSplitPane(int orientation) {
		JSplitPane split = new JSplitPane(orientation);
		// remove the border from the split pane
		split.setBorder(null);
         
		// set the divider size for a more reasonable, less bulky look 
		split.setDividerSize(3);

		// check the UI.  If we can't work with the UI any further, then
		// exit here.
		if (!(split.getUI() instanceof BasicSplitPaneUI))
		   return split;

		//  grab the divider from the UI and remove the border from it
		BasicSplitPaneDivider divider =
					   ((BasicSplitPaneUI) split.getUI()).getDivider();
		if (divider != null)
		   divider.setBorder(null);

		return split;
	}

	static protected void updateObjectGraph(JTree objectGraph, Object o, Session session) {
    	if(session.contains(o)) {
    		BaseNode node = new NodeFactory(session.getSessionFactory()).createObjectNode(session, o);
		
    		objectGraph.setModel(new DefaultTreeModel(node));
    	} else {
    		objectGraph.setModel(new DefaultTreeModel(null));
    	}
	}

	public static AbstractAction makeInspectAction(final JTable fTable, final Session session) {
		return new AbstractAction() {
			public void actionPerformed(ActionEvent ae) {
				int row = fTable.getSelectedRow();
				int col = fTable.getSelectedColumn();
				if (row >= 0 && col >= 0) {
					final Object o = fTable.getModel().getValueAt(row, col);
					Inspector.basicInspect(o);
				}
			}
		};
	}

    // 	doubleclick and ENTER should do the same.
	public static void configureTableAction(final JTable table, final Action actionWhenItemActivated, final Action actionWhenSelectionChanges) {
		if(actionWhenItemActivated!=null) { 
			table.registerKeyboardAction(actionWhenItemActivated, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), JComponent.WHEN_FOCUSED);
			
			table.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent ae) {
					if (ae.getClickCount() == 2) {
						actionWhenItemActivated.actionPerformed(new ActionEvent(table, ActionEvent.ACTION_PERFORMED, ""));
					}
				}
			});
		}
		
		if(actionWhenSelectionChanges!=null) {			
			table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e) {
					if(!e.getValueIsAdjusting()) {
						actionWhenSelectionChanges.actionPerformed(new ActionEvent(table, ActionEvent.ACTION_PERFORMED, ""));
					}
				}
			});
		}
	}
	
	SessionView buildObjectGraph() {
		SessionView v = new ObjectGraphView();
		addSessionListener(v);
		return v;
	}
	
	
	private void addSessionListener(SessionView v) {
		sessionListeners.add(v);
		
	}

	SessionView buildPropertySheet() {
		ObjectPropertySheetView v = new ObjectPropertySheetView();
		addSessionListener(v);
		return v;
	} 
	
	
	public void selectObject(Session session, Object o) {
		fireObjectUpdated(session, o);		
	}

	
	
	public void executeHQLQuery(String text) {
		try {
			if (sf == null) {
				reportInfo("Hibernate not yet configured.");
				return;
			}
			Session session = sf.openSession();
			QueryPage qp = new HQLQueryPage(queryCommander.getQueryText());
			qp.setSession(session);

			qp.setId(++execcount);				
			fireQueryPageCreated(qp);			
			fireFactoryUpdated(sf);
		} catch (HibernateException e1) {

			reportException(e1);
            
		}

		
	}

	public void executeJavaQuery(String text) {
		 try {            	
            if (sf == null) {
                reportInfo("Hibernate not yet configured.");
                return;
            }
            Session session = sf.openSession();
            fireSessionCreated(session);
            QueryPage qp = new JavaPage(queryCommander.getQueryText());
            qp.setSession(session);
            qp.setId(++execcount);
            fireQueryPageCreated(qp);            
        } catch (HibernateException e1) {

            reportException(e1);
        }		
	}

	
}
